@@ -100,9 +100,6 @@ void benchmark(const char* comment, const ncnn::Mat& _in, const ncnn::Option& op
     DataReaderFromEmpty dr;
     net.load_model(dr);
 
-    const std::vector<const char*>& input_names = net.input_names();
-    const std::vector<const char*>& output_names = net.output_names();
-
     if (g_enable_cooling_down)
     {
         // sleep 10 seconds for cooling down SOC  :(
@@ -126,8 +123,8 @@ void benchmark(const char* comment, const ncnn::Mat& _in, const ncnn::Option& op
     for (int i = 0; i < g_warmup_loop_count; i++)
     {
         ncnn::Extractor ex = net.create_extractor();
-        ex.input(input_names[0], in);
-        ex.extract(output_names[0], out);
+        ex.input("data", in);
+        ex.extract("output", out);
     }
 
     double time_min = DBL_MAX;
@@ -140,8 +137,8 @@ void benchmark(const char* comment, const ncnn::Mat& _in, const ncnn::Option& op
 
         {
             ncnn::Extractor ex = net.create_extractor();
-            ex.input(input_names[0], in);
-            ex.extract(output_names[0], out);
+            ex.input("data", in);
+            ex.extract("output", out);
         }
 
         double end = ncnn::get_current_time();
@@ -274,8 +271,6 @@ int main(int argc, char** argv)
 
     benchmark("efficientnet_b0", ncnn::Mat(224, 224, 3), opt);
 
-    benchmark("efficientnetv2_b0", ncnn::Mat(224, 224, 3), opt);
-
     benchmark("regnety_400m", ncnn::Mat(224, 224, 3), opt);
 
     benchmark("blazeface", ncnn::Mat(128, 128, 3), opt);
@@ -312,8 +307,6 @@ int main(int argc, char** argv)
 
     benchmark("yolov4-tiny", ncnn::Mat(416, 416, 3), opt);
 
-    benchmark("nanodet_m", ncnn::Mat(320, 320, 3), opt);
-
 #if NCNN_VULKAN
     delete g_blob_vkallocator;
     delete g_staging_vkallocator;
