@@ -29,12 +29,11 @@ Crop_arm::Crop_arm()
 #endif
 #endif // __ARM_NEON
 
-#if NCNN_BF16
     support_bf16_storage = true;
-#endif
 }
 
 #if __ARM_NEON
+#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 static void crop_pack8_neon(const Mat& src, Mat& dst, int top, int left)
 {
     int w = dst.w;
@@ -60,21 +59,21 @@ static void crop_pack8_neon(const Mat& src, Mat& dst, int top, int left)
     }
 }
 
-static void crop_pack8_bf16_fp16s_neon(const Mat& src, Mat& dst, int top, int left)
+static void crop_pack8_fp16_neon(const Mat& src, Mat& dst, int top, int left)
 {
     int w = dst.w;
     int h = dst.h;
     int right = src.w - dst.w - left;
 
-    const unsigned short* ptr = src.row<unsigned short>(top) + left * 8;
-    unsigned short* outptr = dst;
+    const __fp16* ptr = src.row<__fp16>(top) + left * 8;
+    __fp16* outptr = dst;
 
     for (int y = 0; y < h; y++)
     {
         for (int x = 0; x < w; x++)
         {
-            uint16x8_t _p = vld1q_u16(ptr);
-            vst1q_u16(outptr, _p);
+            float16x8_t _p = vld1q_f16(ptr);
+            vst1q_f16(outptr, _p);
             ptr += 8;
             outptr += 8;
         }
@@ -82,6 +81,7 @@ static void crop_pack8_bf16_fp16s_neon(const Mat& src, Mat& dst, int top, int le
         ptr += (left + right) * 8;
     }
 }
+#endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 
 static void crop_pack4_neon(const Mat& src, Mat& dst, int top, int left)
 {
@@ -106,7 +106,7 @@ static void crop_pack4_neon(const Mat& src, Mat& dst, int top, int left)
     }
 }
 
-static void crop_pack4_bf16_fp16s_neon(const Mat& src, Mat& dst, int top, int left)
+static void crop_pack4_bf16_neon(const Mat& src, Mat& dst, int top, int left)
 {
     int w = dst.w;
     int h = dst.h;
@@ -134,38 +134,38 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
 {
     int w = bottom_blob.w;
     int h = bottom_blob.h;
-    int d = bottom_blob.d;
     int channels = bottom_blob.c;
     int dims = bottom_blob.dims;
     size_t elemsize = bottom_blob.elemsize;
     int elempack = bottom_blob.elempack;
 
 #if __ARM_NEON
+#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
     if (elempack == 8)
     {
-        int _woffset, _hoffset, _doffset, _coffset;
-        int _outw, _outh, _outd, _outc;
-        resolve_crop_roi(bottom_blob.shape(), _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+        int _woffset, _hoffset, _coffset;
+        int _outw, _outh, _outc;
+        resolve_crop_roi(bottom_blob.shape(), _woffset, _hoffset, _coffset, _outw, _outh, _outc);
 
         if (dims == 1)
         {
             int out_elempack = _outw % 8 == 0 ? 8 : _outw % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw / out_elempack == w && out_elempack == 8)
+            if (_outw / out_elempack == w)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_woffset % 8 == 0 && out_elempack == 8)
             {
-                top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 16u)
-                    crop_pack8_bf16_fp16s_neon(bottom_blob, top_blob, 0, _woffset / elempack);
+                    crop_pack8_fp16_neon(bottom_blob, top_blob, 0, _woffset / elempack);
                 else
                     crop_pack8_neon(bottom_blob, top_blob, 0, _woffset / elempack);
 
@@ -178,20 +178,20 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
             int out_elempack = _outh % 8 == 0 ? 8 : _outh % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh / out_elempack == h && out_elempack == 8)
+            if (_outw == w && _outh / out_elempack == h)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_hoffset % 8 == 0 && out_elempack == 8)
             {
-                top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 16u)
-                    crop_pack8_bf16_fp16s_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
+                    crop_pack8_fp16_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
                 else
                     crop_pack8_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
 
@@ -204,12 +204,6 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
             int out_elempack = _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh == h && _outc / out_elempack == channels && out_elempack == 8)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
             if (_coffset % 8 == 0 && out_elempack == 8)
             {
                 const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
@@ -221,6 +215,12 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                         return -100;
                 }
 
+                if (_outw == w && _outh == h && _outc / out_elempack == channels)
+                {
+                    top_blob = bottom_blob;
+                    return 0;
+                }
+
                 top_blob.create(_outw, _outh, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
                 if (top_blob.empty())
                     return -100;
@@ -232,7 +232,7 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                     Mat borderm = top_blob.channel(q);
 
                     if (elemsize == 16u)
-                        crop_pack8_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
+                        crop_pack8_fp16_neon(m, borderm, _hoffset, _woffset);
                     else
                         crop_pack8_neon(m, borderm, _hoffset, _woffset);
                 }
@@ -240,58 +240,14 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                 return 0;
             }
         }
-
-        if (dims == 4)
-        {
-            int out_elempack = _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
-            size_t out_elemsize = elemsize / elempack * out_elempack;
-
-            if (_outw == w && _outh == h && _outd == d && _outc / out_elempack == channels && out_elempack == 8)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
-            if (_coffset % 8 == 0 && out_elempack == 8)
-            {
-                const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
-
-                if (_outw == w && _outh == h && _outd == d)
-                {
-                    top_blob = bottom_blob_sliced.clone();
-                    if (top_blob.empty())
-                        return -100;
-                }
-
-                top_blob.create(_outw, _outh, _outd, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
-                #pragma omp parallel for num_threads(opt.num_threads)
-                for (int q = 0; q < top_blob.c; q++)
-                {
-                    for (int z = 0; z < _outd; z++)
-                    {
-                        const Mat m = bottom_blob_sliced.channel(q).depth(z + _doffset);
-                        Mat borderm = top_blob.channel(q).depth(z);
-
-                        if (elemsize == 16u)
-                            crop_pack8_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
-                        else
-                            crop_pack8_neon(m, borderm, _hoffset, _woffset);
-                    }
-                }
-
-                return 0;
-            }
-        }
     }
+#endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 
     if (elempack == 4)
     {
-        int _woffset, _hoffset, _doffset, _coffset;
-        int _outw, _outh, _outd, _outc;
-        resolve_crop_roi(bottom_blob.shape(), _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+        int _woffset, _hoffset, _coffset;
+        int _outw, _outh, _outc;
+        resolve_crop_roi(bottom_blob.shape(), _woffset, _hoffset, _coffset, _outw, _outh, _outc);
 
         if (dims == 1)
         {
@@ -302,20 +258,20 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw / out_elempack == w && out_elempack == 4)
+            if (_outw / out_elempack == w)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_woffset % 4 == 0 && out_elempack == 4)
             {
-                top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 8u)
-                    crop_pack4_bf16_fp16s_neon(bottom_blob, top_blob, 0, _woffset / elempack);
+                    crop_pack4_bf16_neon(bottom_blob, top_blob, 0, _woffset / elempack);
                 else
                     crop_pack4_neon(bottom_blob, top_blob, 0, _woffset / elempack);
 
@@ -332,20 +288,20 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh / out_elempack == h && out_elempack == 4)
+            if (_outw == w && _outh / out_elempack == h)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_hoffset % 4 == 0 && out_elempack == 4)
             {
-                top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 8u)
-                    crop_pack4_bf16_fp16s_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
+                    crop_pack4_bf16_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
                 else
                     crop_pack4_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
 
@@ -362,12 +318,6 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh == h && _outc / out_elempack == channels && out_elempack == 4)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
             if (_coffset % 4 == 0 && out_elempack == 4)
             {
                 const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
@@ -379,6 +329,12 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                         return -100;
                 }
 
+                if (_outw == w && _outh == h && _outc / out_elempack == channels)
+                {
+                    top_blob = bottom_blob;
+                    return 0;
+                }
+
                 top_blob.create(_outw, _outh, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
                 if (top_blob.empty())
                     return -100;
@@ -390,7 +346,7 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                     Mat borderm = top_blob.channel(q);
 
                     if (elemsize == 8u)
-                        crop_pack4_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
+                        crop_pack4_bf16_neon(m, borderm, _hoffset, _woffset);
                     else
                         crop_pack4_neon(m, borderm, _hoffset, _woffset);
                 }
@@ -398,55 +354,6 @@ int Crop_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt)
                 return 0;
             }
         }
-
-        if (dims == 4)
-        {
-#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
-            int out_elempack = opt.use_fp16_arithmetic && _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
-#else
-            int out_elempack = _outc % 4 == 0 ? 4 : 1;
-#endif
-            size_t out_elemsize = elemsize / elempack * out_elempack;
-
-            if (_outw == w && _outh == h && _outd == d && _outc / out_elempack == channels && out_elempack == 4)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
-            if (_coffset % 4 == 0 && out_elempack == 4)
-            {
-                const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
-
-                if (_outw == w && _outh == h && _outd == d)
-                {
-                    top_blob = bottom_blob_sliced.clone();
-                    if (top_blob.empty())
-                        return -100;
-                }
-
-                top_blob.create(_outw, _outh, _outd, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
-                #pragma omp parallel for num_threads(opt.num_threads)
-                for (int q = 0; q < top_blob.c; q++)
-                {
-                    for (int z = 0; z < _outd; z++)
-                    {
-                        const Mat m = bottom_blob_sliced.channel(q).depth(z + _doffset);
-                        Mat borderm = top_blob.channel(q).depth(z);
-
-                        if (elemsize == 8u)
-                            crop_pack4_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
-                        else
-                            crop_pack4_neon(m, borderm, _hoffset, _woffset);
-                    }
-                }
-
-                return 0;
-            }
-        }
     }
 #endif // __ARM_NEON
 
@@ -469,7 +376,6 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
 
     int w = bottom_blob.w;
     int h = bottom_blob.h;
-    int d = bottom_blob.d;
     int channels = bottom_blob.c;
     int dims = bottom_blob.dims;
     size_t elemsize = bottom_blob.elemsize;
@@ -480,17 +386,18 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
     Mat& top_blob = top_blobs[0];
 
 #if __ARM_NEON
+#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
     if (elempack == 8)
     {
-        int _woffset, _hoffset, _doffset, _coffset;
-        int _outw, _outh, _outd, _outc;
+        int _woffset, _hoffset, _coffset;
+        int _outw, _outh, _outc;
         if (woffset == -233)
         {
-            resolve_crop_roi(bottom_blob.shape(), (const int*)reference_blob, _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+            resolve_crop_roi(bottom_blob.shape(), (const int*)reference_blob, _woffset, _hoffset, _coffset, _outw, _outh, _outc);
         }
         else
         {
-            resolve_crop_roi(bottom_blob.shape(), reference_blob.shape(), _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+            resolve_crop_roi(bottom_blob.shape(), reference_blob.shape(), _woffset, _hoffset, _coffset, _outw, _outh, _outc);
         }
 
         if (dims == 1)
@@ -498,20 +405,20 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
             int out_elempack = _outw % 8 == 0 ? 8 : _outw % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw / out_elempack == w && out_elempack == 8)
+            if (_outw / out_elempack == w)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_woffset % 8 == 0 && out_elempack == 8)
             {
-                top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 16u)
-                    crop_pack8_bf16_fp16s_neon(bottom_blob, top_blob, 0, _woffset / elempack);
+                    crop_pack8_fp16_neon(bottom_blob, top_blob, 0, _woffset / elempack);
                 else
                     crop_pack8_neon(bottom_blob, top_blob, 0, _woffset / elempack);
 
@@ -524,20 +431,20 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
             int out_elempack = _outh % 8 == 0 ? 8 : _outh % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh / out_elempack == h && out_elempack == 8)
+            if (_outw == w && _outh / out_elempack == h)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_hoffset % 8 == 0 && out_elempack == 8)
             {
-                top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 16u)
-                    crop_pack8_bf16_fp16s_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
+                    crop_pack8_fp16_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
                 else
                     crop_pack8_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
 
@@ -550,12 +457,6 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
             int out_elempack = _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh == h && _outc / out_elempack == channels && out_elempack == 8)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
             if (_coffset % 8 == 0 && out_elempack == 8)
             {
                 const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
@@ -567,6 +468,12 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                         return -100;
                 }
 
+                if (_outw == w && _outh == h && _outc / out_elempack == channels)
+                {
+                    top_blob = bottom_blob;
+                    return 0;
+                }
+
                 top_blob.create(_outw, _outh, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
                 if (top_blob.empty())
                     return -100;
@@ -578,7 +485,7 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                     Mat borderm = top_blob.channel(q);
 
                     if (elemsize == 16u)
-                        crop_pack8_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
+                        crop_pack8_fp16_neon(m, borderm, _hoffset, _woffset);
                     else
                         crop_pack8_neon(m, borderm, _hoffset, _woffset);
                 }
@@ -586,64 +493,20 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                 return 0;
             }
         }
-
-        if (dims == 4)
-        {
-            int out_elempack = _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
-            size_t out_elemsize = elemsize / elempack * out_elempack;
-
-            if (_outw == w && _outh == h && _outd == d && _outc / out_elempack == channels && out_elempack == 8)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
-            if (_coffset % 8 == 0 && out_elempack == 8)
-            {
-                const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
-
-                if (_outw == w && _outh == h && _outd == d)
-                {
-                    top_blob = bottom_blob_sliced.clone();
-                    if (top_blob.empty())
-                        return -100;
-                }
-
-                top_blob.create(_outw, _outh, _outd, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
-                #pragma omp parallel for num_threads(opt.num_threads)
-                for (int q = 0; q < top_blob.c; q++)
-                {
-                    for (int z = 0; z < _outd; z++)
-                    {
-                        const Mat m = bottom_blob_sliced.channel(q).depth(z + _doffset);
-                        Mat borderm = top_blob.channel(q).depth(z);
-
-                        if (elemsize == 16u)
-                            crop_pack8_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
-                        else
-                            crop_pack8_neon(m, borderm, _hoffset, _woffset);
-                    }
-                }
-
-                return 0;
-            }
-        }
     }
+#endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 
     if (elempack == 4)
     {
-        int _woffset, _hoffset, _doffset, _coffset;
-        int _outw, _outh, _outd, _outc;
+        int _woffset, _hoffset, _coffset;
+        int _outw, _outh, _outc;
         if (woffset == -233)
         {
-            resolve_crop_roi(bottom_blob.shape(), (const int*)reference_blob, _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+            resolve_crop_roi(bottom_blob.shape(), (const int*)reference_blob, _woffset, _hoffset, _coffset, _outw, _outh, _outc);
         }
         else
         {
-            resolve_crop_roi(bottom_blob.shape(), reference_blob.shape(), _woffset, _hoffset, _doffset, _coffset, _outw, _outh, _outd, _outc);
+            resolve_crop_roi(bottom_blob.shape(), reference_blob.shape(), _woffset, _hoffset, _coffset, _outw, _outh, _outc);
         }
 
         if (dims == 1)
@@ -655,20 +518,20 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw / out_elempack == w && out_elempack == 4)
+            if (_outw / out_elempack == w)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_woffset % 4 == 0 && out_elempack == 4)
             {
-                top_blob.create(_outw / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 8u)
-                    crop_pack4_bf16_fp16s_neon(bottom_blob, top_blob, 0, _woffset / elempack);
+                    crop_pack4_bf16_neon(bottom_blob, top_blob, 0, _woffset / elempack);
                 else
                     crop_pack4_neon(bottom_blob, top_blob, 0, _woffset / elempack);
 
@@ -685,20 +548,20 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh / out_elempack == h && out_elempack == 4)
+            if (_outw == w && _outh / out_elempack == h)
             {
                 top_blob = bottom_blob;
                 return 0;
             }
 
+            top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
+            if (top_blob.empty())
+                return -100;
+
             if (_hoffset % 4 == 0 && out_elempack == 4)
             {
-                top_blob.create(_outw, _outh / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
                 if (elemsize == 8u)
-                    crop_pack4_bf16_fp16s_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
+                    crop_pack4_bf16_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
                 else
                     crop_pack4_neon(bottom_blob, top_blob, _hoffset / elempack, _woffset);
 
@@ -715,12 +578,6 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
 #endif
             size_t out_elemsize = elemsize / elempack * out_elempack;
 
-            if (_outw == w && _outh == h && _outc / out_elempack == channels && out_elempack == 4)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
             if (_coffset % 4 == 0 && out_elempack == 4)
             {
                 const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
@@ -732,6 +589,12 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                         return -100;
                 }
 
+                if (_outw == w && _outh == h && _outc / out_elempack == channels)
+                {
+                    top_blob = bottom_blob;
+                    return 0;
+                }
+
                 top_blob.create(_outw, _outh, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
                 if (top_blob.empty())
                     return -100;
@@ -743,7 +606,7 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                     Mat borderm = top_blob.channel(q);
 
                     if (elemsize == 8u)
-                        crop_pack4_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
+                        crop_pack4_bf16_neon(m, borderm, _hoffset, _woffset);
                     else
                         crop_pack4_neon(m, borderm, _hoffset, _woffset);
                 }
@@ -751,55 +614,6 @@ int Crop_arm::forward(const std::vector<Mat>& bottom_blobs, std::vector<Mat>& to
                 return 0;
             }
         }
-
-        if (dims == 4)
-        {
-#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
-            int out_elempack = opt.use_fp16_arithmetic && _outc % 8 == 0 ? 8 : _outc % 4 == 0 ? 4 : 1;
-#else
-            int out_elempack = _outc % 4 == 0 ? 4 : 1;
-#endif
-            size_t out_elemsize = elemsize / elempack * out_elempack;
-
-            if (_outw == w && _outh == h && _outd == d && _outc / out_elempack == channels && out_elempack == 4)
-            {
-                top_blob = bottom_blob;
-                return 0;
-            }
-
-            if (_coffset % 4 == 0 && out_elempack == 4)
-            {
-                const Mat bottom_blob_sliced = bottom_blob.channel_range(_coffset / out_elempack, _outc / out_elempack);
-
-                if (_outw == w && _outh == h && _outd == d)
-                {
-                    top_blob = bottom_blob_sliced.clone();
-                    if (top_blob.empty())
-                        return -100;
-                }
-
-                top_blob.create(_outw, _outh, _outd, _outc / out_elempack, out_elemsize, out_elempack, opt.blob_allocator);
-                if (top_blob.empty())
-                    return -100;
-
-                #pragma omp parallel for num_threads(opt.num_threads)
-                for (int q = 0; q < top_blob.c; q++)
-                {
-                    for (int z = 0; z < _outd; z++)
-                    {
-                        const Mat m = bottom_blob_sliced.channel(q).depth(z + _doffset);
-                        Mat borderm = top_blob.channel(q).depth(z);
-
-                        if (elemsize == 8u)
-                            crop_pack4_bf16_fp16s_neon(m, borderm, _hoffset, _woffset);
-                        else
-                            crop_pack4_neon(m, borderm, _hoffset, _woffset);
-                    }
-                }
-
-                return 0;
-            }
-        }
     }
 #endif // __ARM_NEON
 
