@@ -18,10 +18,16 @@
 
 #if __ARM_NEON
 #include <arm_neon.h>
+#include "neon_mathfun.h"
+#if __aarch64__
+#include "gemm_symm_int8.h"
+#endif
+#if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
+#include "neon_mathfun_fp16s.h"
+#endif
 #endif // __ARM_NEON
-
-#include "arm_activation.h"
-#include "arm_usability.h"
+#include "cpu.h"
+#include "neon_activation.h"
 
 namespace ncnn {
 
@@ -34,16 +40,15 @@ InnerProduct_arm::InnerProduct_arm()
 #endif
 #endif // __ARM_NEON
 
-#if NCNN_BF16
     support_bf16_storage = true;
-#endif
 
     flatten = 0;
-    activation = 0;
 }
 
 int InnerProduct_arm::create_pipeline(const Option& opt)
 {
+#if __ARM_NEON
+    if (opt.use_packing_layout or opt.use_int8_inference)
     {
         flatten = ncnn::create_layer(ncnn::LayerType::Flatten);
 
@@ -53,13 +58,7 @@ int InnerProduct_arm::create_pipeline(const Option& opt)
 
         flatten->create_pipeline(opt);
     }
-
-#if NCNN_INT8
-    if (opt.use_int8_inference && weight_data.elemsize == (size_t)1u)
-    {
-        return create_pipeline_int8_arm(opt);
-    }
-#endif
+#endif // __ARM_NEON
 
 #if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
     if (opt.use_fp16_storage)
@@ -68,12 +67,15 @@ int InnerProduct_arm::create_pipeline(const Option& opt)
     }
 #endif
 
-#if NCNN_BF16
     if (opt.use_bf16_storage)
     {
         return create_pipeline_bf16s(opt);
     }
-#endif
+
+    if (opt.use_int8_inference)
+    {
+        return create_pipeline_int8(opt);
+    }
 
     return 0;
 }
@@ -87,24 +89,119 @@ int InnerProduct_arm::destroy_pipeline(const Option& opt)
         flatten = 0;
     }
 
-    if (activation)
+    return 0;
+}
+
+int InnerProduct_arm::create_pipeline_int8(const Option& opt)
+{
+    // convert fp32 to int8
+    if (weight_data_int8_scales.empty())
     {
-        activation->destroy_pipeline(opt);
-        delete activation;
-        activation = 0;
+        return 0;
+    }
+#if __aarch64__
+    // first reorder Matrix A before MatMul
+    const int n = num_output;
+    const int k = weight_data.total() / n;
+    weight_data_int8.create(n * k, (size_t)1u, opt.blob_allocator);
+
+    int8_t* b = weight_data;
+    int8_t* sb = weight_data_int8;
+    reorder_a(b, sb, n, k, k);
+
+    // pre-built scales
+    scales_in.create(num_output, 4u, opt.blob_allocator);
+    for (int i = 0; i < num_output; ++i)
+    {
+        if (std::fabs(static_cast<float>(weight_data_int8_scales[i])) <= 1e-6)
+        {
+            scales_in[i] = 0.f;
+        }
+        else
+        {
+            scales_in[i] = 1.f / (bottom_blob_int8_scale * weight_data_int8_scales[i]);
+        }
+    }
+#endif
+    return 0;
+}
+
+int InnerProduct_arm::forward_int8(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const
+{
+    const int num_input = weight_data_size / num_output;
+
+    if (bottom_blob.dims == 2 && bottom_blob.w == num_input && bottom_blob.h > 1)
+    {
+        return InnerProduct::forward(bottom_blob, top_blob, opt);
     }
 
+#if __aarch64__
+    Mat bottom_blob_tm = bottom_blob;
+    if (bottom_blob.elemsize != 1)
+    {
+        quantize_float32_to_int8(bottom_blob, bottom_blob_tm, bottom_blob_int8_scale, opt);
+    }
+
+    Mat bottom_blob_tm_flattened = bottom_blob_tm;
+    if (bottom_blob_tm.dims != 1)
+    {
+        Option opt_flatten = opt;
+        opt_flatten.blob_allocator = opt.workspace_allocator;
+        flatten->forward(bottom_blob_tm, bottom_blob_tm_flattened, opt_flatten);
+    }
+
+    top_blob.create(num_output, 4u, opt.blob_allocator);
+    if (top_blob.empty())
+    {
+        return -100;
+    }
+
+    const int w = bottom_blob_tm.w;
+    const int h = bottom_blob_tm.h;
+
+    const int m = 1;
+    const int k = bottom_blob_tm.c * w * h;
+    Mat bottom_blob_reorder(m * k, (size_t)1u, opt.workspace_allocator);
+    {
+        reorder_a(bottom_blob_tm_flattened, bottom_blob_reorder, m, k, k);
+    }
+
+    Mat top_blob_tm(m * num_output, (size_t)4u, opt.workspace_allocator);
+    int32_t* pc = top_blob_tm;
+    const int8_t* pa = bottom_blob_reorder;
+    const int8_t* pb = weight_data_int8;
+    int8kernel((void*)pc, pa, pb, m, k, num_output, num_output, 0, 0, opt);
+
+    float* outptr = top_blob;
+
+    // dequant.fused.relu int32_t to float
+    for (int p = 0; p < num_output; ++p)
+    {
+        float sumfp32 = pc[p] * scales_in[p];
+        if (bias_term)
+        {
+            sumfp32 += bias_data[p];
+        }
+        if (1 == activation_type)
+        {
+            sumfp32 = std::max(0.f, sumfp32);
+        }
+
+        outptr[p] = sumfp32;
+    }
     return 0;
+#else
+    return InnerProduct::forward(bottom_blob, top_blob, opt);
+#endif
 }
 
 int InnerProduct_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const
 {
-#if NCNN_INT8
     if (opt.use_int8_inference && weight_data.elemsize == (size_t)1u)
     {
-        return forward_int8_arm(bottom_blob, top_blob, opt);
+        // TODO
+        return forward_int8(bottom_blob, top_blob, opt);
     }
-#endif
 
     int elembits = bottom_blob.elembits();
 
@@ -118,10 +215,8 @@ int InnerProduct_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Optio
     }
 #endif
 
-#if NCNN_BF16
     if (opt.use_bf16_storage && elembits == 16)
         return forward_bf16s(bottom_blob, top_blob, opt);
-#endif
 
     const int num_input = weight_data_size / num_output;
 
@@ -195,7 +290,30 @@ int InnerProduct_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Optio
                         sum += m[i] * kptr[i];
                     }
 
-                    sum = activation_ss(sum, activation_type, activation_params);
+                    if (activation_type == 1)
+                    {
+                        sum = std::max(sum, 0.f);
+                    }
+                    else if (activation_type == 2)
+                    {
+                        float slope = activation_params[0];
+                        sum = sum > 0.f ? sum : sum * slope;
+                    }
+                    else if (activation_type == 3)
+                    {
+                        float min = activation_params[0];
+                        float max = activation_params[1];
+                        if (sum < min) sum = min;
+                        if (sum > max) sum = max;
+                    }
+                    else if (activation_type == 4)
+                    {
+                        sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                    }
+                    else if (activation_type == 5)
+                    {
+                        sum = static_cast<float>(sum * tanh(log(exp(sum) + 1.f)));
+                    }
 
                     outptr[0] = sum;
                     outptr += 1;
@@ -344,10 +462,48 @@ int InnerProduct_arm::forward(const Mat& bottom_blob, Mat& top_blob, const Optio
 
 #endif // __ARM_NEON
 
-        sum0 = activation_ss(sum0, activation_type, activation_params);
-        sum1 = activation_ss(sum1, activation_type, activation_params);
-        sum2 = activation_ss(sum2, activation_type, activation_params);
-        sum3 = activation_ss(sum3, activation_type, activation_params);
+        if (activation_type == 1)
+        {
+            sum0 = std::max(sum0, 0.f);
+            sum1 = std::max(sum1, 0.f);
+            sum2 = std::max(sum2, 0.f);
+            sum3 = std::max(sum3, 0.f);
+        }
+        else if (activation_type == 2)
+        {
+            float slope = activation_params[0];
+            sum0 = sum0 > 0.f ? sum0 : sum0 * slope;
+            sum1 = sum1 > 0.f ? sum1 : sum1 * slope;
+            sum2 = sum2 > 0.f ? sum2 : sum2 * slope;
+            sum3 = sum3 > 0.f ? sum3 : sum3 * slope;
+        }
+        else if (activation_type == 3)
+        {
+            float min = activation_params[0];
+            float max = activation_params[1];
+            if (sum0 < min) sum0 = min;
+            if (sum0 > max) sum0 = max;
+            if (sum1 < min) sum1 = min;
+            if (sum1 > max) sum1 = max;
+            if (sum2 < min) sum2 = min;
+            if (sum2 > max) sum2 = max;
+            if (sum3 < min) sum3 = min;
+            if (sum3 > max) sum3 = max;
+        }
+        else if (activation_type == 4)
+        {
+            sum0 = static_cast<float>(1.f / (1.f + exp(-sum0)));
+            sum1 = static_cast<float>(1.f / (1.f + exp(-sum1)));
+            sum2 = static_cast<float>(1.f / (1.f + exp(-sum2)));
+            sum3 = static_cast<float>(1.f / (1.f + exp(-sum3)));
+        }
+        else if (activation_type == 5)
+        {
+            sum0 = static_cast<float>(sum0 * tanh(log(exp(sum0) + 1.f)));
+            sum1 = static_cast<float>(sum1 * tanh(log(exp(sum1) + 1.f)));
+            sum2 = static_cast<float>(sum2 * tanh(log(exp(sum2) + 1.f)));
+            sum3 = static_cast<float>(sum3 * tanh(log(exp(sum3) + 1.f)));
+        }
 
         top_blob[p] = sum0;
         top_blob[p + 1] = sum1;
@@ -661,7 +817,30 @@ int InnerProduct_arm::forward_fp16s(const Mat& bottom_blob, Mat& top_blob, const
                         sum += (float)m[i] * (float)kptr[i];
                     }
 
-                    sum = activation_ss(sum, activation_type, activation_params);
+                    if (activation_type == 1)
+                    {
+                        sum = std::max(sum, 0.f);
+                    }
+                    else if (activation_type == 2)
+                    {
+                        float slope = activation_params[0];
+                        sum = sum > 0.f ? sum : sum * slope;
+                    }
+                    else if (activation_type == 3)
+                    {
+                        float min = activation_params[0];
+                        float max = activation_params[1];
+                        if (sum < min) sum = min;
+                        if (sum > max) sum = max;
+                    }
+                    else if (activation_type == 4)
+                    {
+                        sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                    }
+                    else if (activation_type == 5)
+                    {
+                        sum = static_cast<float>(sum * tanh(log(exp(sum) + 1.f)));
+                    }
 
                     outptr[0] = (__fp16)sum;
                     outptr += 1;
@@ -1205,7 +1384,30 @@ int InnerProduct_arm::forward_fp16sa(const Mat& bottom_blob, Mat& top_blob, cons
                         sum += (float)(m[i] * kptr[i]);
                     }
 
-                    sum = activation_ss(sum, activation_type, activation_params);
+                    if (activation_type == 1)
+                    {
+                        sum = std::max(sum, 0.f);
+                    }
+                    else if (activation_type == 2)
+                    {
+                        float slope = activation_params[0];
+                        sum = sum > 0.f ? sum : sum * slope;
+                    }
+                    else if (activation_type == 3)
+                    {
+                        float min = activation_params[0];
+                        float max = activation_params[1];
+                        if (sum < min) sum = min;
+                        if (sum > max) sum = max;
+                    }
+                    else if (activation_type == 4)
+                    {
+                        sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                    }
+                    else if (activation_type == 5)
+                    {
+                        sum = static_cast<float>(sum * tanh(log(exp(sum) + 1.f)));
+                    }
 
                     outptr[0] = (__fp16)sum;
                     outptr += 1;
@@ -1538,7 +1740,6 @@ int InnerProduct_arm::forward_fp16sa(const Mat& bottom_blob, Mat& top_blob, cons
 }
 #endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 
-#if NCNN_BF16
 int InnerProduct_arm::create_pipeline_bf16s(const Option& opt)
 {
     const int num_input = weight_data_size / num_output;
@@ -1739,7 +1940,30 @@ int InnerProduct_arm::forward_bf16s(const Mat& bottom_blob, Mat& top_blob, const
                         sum += bfloat16_to_float32(m[i]) * bfloat16_to_float32(kptr[i]);
                     }
 
-                    sum = activation_ss(sum, activation_type, activation_params);
+                    if (activation_type == 1)
+                    {
+                        sum = std::max(sum, 0.f);
+                    }
+                    else if (activation_type == 2)
+                    {
+                        float slope = activation_params[0];
+                        sum = sum > 0.f ? sum : sum * slope;
+                    }
+                    else if (activation_type == 3)
+                    {
+                        float min = activation_params[0];
+                        float max = activation_params[1];
+                        if (sum < min) sum = min;
+                        if (sum > max) sum = max;
+                    }
+                    else if (activation_type == 4)
+                    {
+                        sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                    }
+                    else if (activation_type == 5)
+                    {
+                        sum = static_cast<float>(sum * tanh(log(exp(sum) + 1.f)));
+                    }
 
                     outptr[0] = float32_to_bfloat16(sum);
                     outptr += 1;
@@ -1899,485 +2123,5 @@ int InnerProduct_arm::forward_bf16s(const Mat& bottom_blob, Mat& top_blob, const
 
     return 0;
 }
-#endif // NCNN_BF16
-
-#if NCNN_INT8
-int InnerProduct_arm::create_pipeline_int8_arm(const Option& opt)
-{
-    activation = create_activation_layer(activation_type, activation_params, opt);
-
-    const int num_input = weight_data_size / num_output;
-
-    int out_elempack = 1;
-#if __ARM_NEON
-    if (opt.use_packing_layout)
-    {
-        out_elempack = num_output % 8 == 0 ? 8 : 1;
-    }
-#endif
-
-    // src = inch-outch
-    // dst = pb-inch-outch/pb
-    {
-        Mat weight_data_r2 = weight_data.reshape(num_input, num_output);
-
-        weight_data_int8.create(num_input, num_output / out_elempack, (size_t)out_elempack, out_elempack);
-
-        for (int q = 0; q + (out_elempack - 1) < num_output; q += out_elempack)
-        {
-            signed char* g0 = weight_data_int8.row<signed char>(q / out_elempack);
-
-            for (int p = 0; p < num_input; p++)
-            {
-                for (int j = 0; j < out_elempack; j++)
-                {
-                    *g0++ = weight_data_r2.row<signed char>(q + j)[p];
-                }
-            }
-        }
-    }
-
-    return 0;
-}
-
-int InnerProduct_arm::forward_int8_arm(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const
-{
-    const int num_input = weight_data_size / num_output;
-
-    int elembits = bottom_blob.elembits();
-
-    Mat bottom_blob_int8 = bottom_blob;
-    if (elembits != 8)
-    {
-        Option opt_q = opt;
-        opt_q.blob_allocator = opt.workspace_allocator;
-        quantize_to_int8(bottom_blob, bottom_blob_int8, bottom_blob_int8_scales, opt_q);
-    }
-
-    if (bottom_blob_int8.dims == 2 && bottom_blob_int8.w == num_input && bottom_blob_int8.h * bottom_blob_int8.elempack > 1)
-    {
-        // gemm
-        int h = bottom_blob_int8.h;
-        int elempack = bottom_blob_int8.elempack;
-
-        int out_elempack = 1;
-#if __ARM_NEON
-        if (opt.use_packing_layout)
-        {
-            out_elempack = h * elempack % 4 == 0 ? 4 : 1;
-        }
-#endif
-
-        int outh = h * elempack / out_elempack;
-
-        top_blob.create(num_output, outh, (size_t)(4u * out_elempack), out_elempack, opt.blob_allocator);
-        if (top_blob.empty())
-            return -100;
-
-        Mat scale_data(num_output);
-        for (int p = 0; p < num_output; p++)
-        {
-            // dequantize
-            float scale_in;
-            if (weight_data_int8_scales[p] == 0)
-                scale_in = 0;
-            else
-                scale_in = 1.f / (bottom_blob_int8_scales[0] * weight_data_int8_scales[p]);
-
-            scale_data[p] = scale_in;
-        }
-
-#if __ARM_NEON
-        if (elempack == 8)
-        {
-            #pragma omp parallel for num_threads(opt.num_threads)
-            for (int j = 0; j < h; j++)
-            {
-                float* outptr0 = top_blob.row(j * 2);
-                float* outptr1 = top_blob.row(j * 2 + 1);
-
-                for (int p = 0; p < num_output; p++)
-                {
-                    const signed char* kptr = (const signed char*)weight_data + num_input * p;
-                    const signed char* m = bottom_blob_int8.row<const signed char>(j);
-
-                    int32x4_t _sum0 = vdupq_n_s32(0);
-                    int32x4_t _sum1 = vdupq_n_s32(0);
-
-                    int i = 0;
-                    for (; i + 3 < num_input; i += 4)
-                    {
-                        int8x16_t _val0 = vld1q_s8(m);
-                        int8x16_t _val1 = vld1q_s8(m + 16);
-
-                        int8x8_t _w0 = vdup_n_s8(kptr[0]);
-                        int8x8_t _w1 = vdup_n_s8(kptr[1]);
-                        int8x8_t _w2 = vdup_n_s8(kptr[2]);
-                        int8x8_t _w3 = vdup_n_s8(kptr[3]);
-
-                        int16x8_t _s0 = vmull_s8(vget_low_s8(_val0), _w0);
-                        int16x8_t _s1 = vmull_s8(vget_low_s8(_val1), _w2);
-                        _s0 = vmlal_s8(_s0, vget_high_s8(_val0), _w1);
-                        _s1 = vmlal_s8(_s1, vget_high_s8(_val1), _w3);
-
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s1));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s1));
-
-                        m += 32;
-                        kptr += 4;
-                    }
-                    for (; i + 1 < num_input; i += 2)
-                    {
-                        int8x16_t _val0 = vld1q_s8(m);
-
-                        int8x8_t _w0 = vdup_n_s8(kptr[0]);
-                        int8x8_t _w1 = vdup_n_s8(kptr[1]);
-
-                        int16x8_t _s0 = vmull_s8(vget_low_s8(_val0), _w0);
-                        _s0 = vmlal_s8(_s0, vget_high_s8(_val0), _w1);
-
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-
-                        m += 16;
-                        kptr += 2;
-                    }
-                    for (; i < num_input; i++)
-                    {
-                        int8x8_t _val = vld1_s8(m);
-                        int8x8_t _w = vdup_n_s8(kptr[0]);
-
-                        int16x8_t _s0 = vmull_s8(_val, _w);
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-
-                        m += 8;
-                        kptr += 1;
-                    }
-
-                    // dequantize and relu
-                    float32x4_t _scale_in = vdupq_n_f32(scale_data[p]);
-
-                    float32x4_t _sumfp32_0 = vcvtq_f32_s32(_sum0);
-                    float32x4_t _sumfp32_1 = vcvtq_f32_s32(_sum1);
-                    if (bias_term)
-                    {
-                        float32x4_t _bias = vdupq_n_f32(bias_data[p]);
-                        _sumfp32_0 = vmlaq_f32(_bias, _sumfp32_0, _scale_in);
-                        _sumfp32_1 = vmlaq_f32(_bias, _sumfp32_1, _scale_in);
-                    }
-                    else
-                    {
-                        _sumfp32_0 = vmulq_f32(_sumfp32_0, _scale_in);
-                        _sumfp32_1 = vmulq_f32(_sumfp32_1, _scale_in);
-                    }
-
-                    _sumfp32_0 = activation_ps(_sumfp32_0, activation_type, activation_params);
-                    _sumfp32_1 = activation_ps(_sumfp32_1, activation_type, activation_params);
-
-                    vst1q_f32(outptr0, _sumfp32_0);
-                    vst1q_f32(outptr1, _sumfp32_1);
-                    outptr0 += 4;
-                    outptr1 += 4;
-                }
-            }
-        }
-
-        if (elempack == 1 && out_elempack == 4)
-        {
-            #pragma omp parallel for num_threads(opt.num_threads)
-            for (int j = 0; j < outh; j++)
-            {
-                float* outptr = top_blob.row(j);
-
-                for (int p = 0; p < num_output; p++)
-                {
-                    const signed char* kptr = (const signed char*)weight_data + num_input * p;
-                    const signed char* m0 = bottom_blob_int8.row<const signed char>(j * 4);
-                    const signed char* m1 = bottom_blob_int8.row<const signed char>(j * 4 + 1);
-                    const signed char* m2 = bottom_blob_int8.row<const signed char>(j * 4 + 2);
-                    const signed char* m3 = bottom_blob_int8.row<const signed char>(j * 4 + 3);
-
-                    int sum0 = 0;
-                    int sum1 = 0;
-                    int sum2 = 0;
-                    int sum3 = 0;
-
-                    int i = 0;
-
-                    int32x4_t _sum0 = vdupq_n_s32(0);
-                    int32x4_t _sum1 = vdupq_n_s32(0);
-                    int32x4_t _sum2 = vdupq_n_s32(0);
-                    int32x4_t _sum3 = vdupq_n_s32(0);
-                    for (; i + 7 < num_input; i += 8)
-                    {
-                        int8x8_t _val0 = vld1_s8(m0);
-                        int8x8_t _val1 = vld1_s8(m1);
-                        int8x8_t _val2 = vld1_s8(m2);
-                        int8x8_t _val3 = vld1_s8(m3);
-                        int8x8_t _w = vld1_s8(kptr);
-
-                        int16x8_t _s0 = vmull_s8(_val0, _w);
-                        int16x8_t _s1 = vmull_s8(_val1, _w);
-                        int16x8_t _s2 = vmull_s8(_val2, _w);
-                        int16x8_t _s3 = vmull_s8(_val3, _w);
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_low_s16(_s1));
-                        _sum2 = vaddw_s16(_sum2, vget_low_s16(_s2));
-                        _sum3 = vaddw_s16(_sum3, vget_low_s16(_s3));
-                        _sum0 = vaddw_s16(_sum0, vget_high_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s1));
-                        _sum2 = vaddw_s16(_sum2, vget_high_s16(_s2));
-                        _sum3 = vaddw_s16(_sum3, vget_high_s16(_s3));
-
-                        m0 += 8;
-                        m1 += 8;
-                        m2 += 8;
-                        m3 += 8;
-                        kptr += 8;
-                    }
-#if __aarch64__
-                    sum0 = vaddvq_s32(_sum0);
-                    sum1 = vaddvq_s32(_sum1);
-                    sum2 = vaddvq_s32(_sum2);
-                    sum3 = vaddvq_s32(_sum3);
-#else
-                    int32x2_t _s20 = vadd_s32(vget_low_s32(_sum0), vget_high_s32(_sum0));
-                    int32x2_t _s21 = vadd_s32(vget_low_s32(_sum1), vget_high_s32(_sum1));
-                    int32x2_t _s22 = vadd_s32(vget_low_s32(_sum2), vget_high_s32(_sum2));
-                    int32x2_t _s23 = vadd_s32(vget_low_s32(_sum3), vget_high_s32(_sum3));
-                    int32x2_t _s201 = vpadd_s32(_s20, _s21);
-                    int32x2_t _s223 = vpadd_s32(_s22, _s23);
-                    sum0 = vget_lane_s32(_s201, 0);
-                    sum1 = vget_lane_s32(_s201, 1);
-                    sum2 = vget_lane_s32(_s223, 0);
-                    sum3 = vget_lane_s32(_s223, 1);
-#endif
-                    for (; i < num_input; i++)
-                    {
-                        sum0 += *m0++ * kptr[0];
-                        sum1 += *m1++ * kptr[0];
-                        sum2 += *m2++ * kptr[0];
-                        sum3 += *m3++ * kptr[0];
-                        kptr += 1;
-                    }
-
-                    // dequantize and relu
-                    float sumfp32_0 = sum0 * scale_data[p];
-                    float sumfp32_1 = sum1 * scale_data[p];
-                    float sumfp32_2 = sum2 * scale_data[p];
-                    float sumfp32_3 = sum3 * scale_data[p];
-
-                    if (bias_term)
-                    {
-                        sumfp32_0 += bias_data[p];
-                        sumfp32_1 += bias_data[p];
-                        sumfp32_2 += bias_data[p];
-                        sumfp32_3 += bias_data[p];
-                    }
-
-                    outptr[0] = activation_ss(sumfp32_0, activation_type, activation_params);
-                    outptr[1] = activation_ss(sumfp32_1, activation_type, activation_params);
-                    outptr[2] = activation_ss(sumfp32_2, activation_type, activation_params);
-                    outptr[3] = activation_ss(sumfp32_3, activation_type, activation_params);
-                    outptr += 4;
-                }
-            }
-        }
-#endif // __ARM_NEON
-
-        if (elempack == 1 && out_elempack == 1)
-        {
-            #pragma omp parallel for num_threads(opt.num_threads)
-            for (int j = 0; j < outh; j++)
-            {
-                float* outptr = top_blob.row(j);
-
-                for (int p = 0; p < num_output; p++)
-                {
-                    const signed char* kptr = (const signed char*)weight_data + num_input * p;
-                    const signed char* m = bottom_blob_int8.row<const signed char>(j);
-
-                    int sum = 0;
-
-                    int i = 0;
-#if __ARM_NEON
-                    int32x4_t _sum0 = vdupq_n_s32(0);
-                    int32x4_t _sum1 = vdupq_n_s32(0);
-                    for (; i + 7 < num_input; i += 8)
-                    {
-                        int8x8_t _val = vld1_s8(m);
-                        int8x8_t _w = vld1_s8(kptr);
-
-                        int16x8_t _s0 = vmull_s8(_val, _w);
-                        _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                        _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-
-                        m += 8;
-                        kptr += 8;
-                    }
-
-                    _sum0 = vaddq_s32(_sum0, _sum1);
-#if __aarch64__
-                    sum = vaddvq_s32(_sum0);
-#else
-                    int32x2_t _s2 = vadd_s32(vget_low_s32(_sum0), vget_high_s32(_sum0));
-                    _s2 = vpadd_s32(_s2, _s2);
-                    sum = vget_lane_s32(_s2, 0);
-#endif
-#endif // __ARM_NEON
-                    for (; i < num_input; i++)
-                    {
-                        sum += *m++ * *kptr++;
-                    }
-
-                    // dequantize and relu
-                    float sumfp32 = sum * scale_data[p];
-
-                    if (bias_term)
-                        sumfp32 += bias_data[p];
-
-                    outptr[0] = activation_ss(sumfp32, activation_type, activation_params);
-                    outptr += 1;
-                }
-            }
-        }
-
-        return 0;
-    }
-
-    Mat bottom_blob_int8_flattened = bottom_blob_int8;
-    if (bottom_blob_int8.dims != 1)
-    {
-        Option opt_flatten = opt;
-        opt_flatten.blob_allocator = opt.workspace_allocator;
-        flatten->forward(bottom_blob_int8, bottom_blob_int8_flattened, opt_flatten);
-    }
-
-    //     int elempack = bottom_blob_int8_flattened.elempack;
-
-    int out_elempack = 1;
-#if __ARM_NEON
-    if (opt.use_packing_layout)
-    {
-        out_elempack = num_output % 8 == 0 ? 8 : 1;
-    }
-#endif
-
-    top_blob.create(num_output / out_elempack, (size_t)(4u * out_elempack), out_elempack, opt.blob_allocator);
-    if (top_blob.empty())
-        return -100;
-
-    Mat top_blob_int32;
-    top_blob_int32.create(num_output / out_elempack, (size_t)(4u * out_elempack), out_elempack, opt.workspace_allocator);
-    if (top_blob_int32.empty())
-        return -100;
-
-#if __ARM_NEON
-    if (out_elempack == 8)
-    {
-        // num_output
-        #pragma omp parallel for num_threads(opt.num_threads)
-        for (int p = 0; p < num_output / out_elempack; p++)
-        {
-            const signed char* kptr = weight_data_int8.row<const signed char>(p);
-            const signed char* sptr = bottom_blob_int8_flattened;
-
-            int32x4_t _sum0 = vdupq_n_s32(0);
-            int32x4_t _sum1 = vdupq_n_s32(0);
-
-            int i = 0;
-            for (; i + 1 < num_input; i += 2)
-            {
-                int8x8_t _val0 = vdup_n_s8(sptr[0]);
-                int8x8_t _val1 = vdup_n_s8(sptr[1]);
-
-                int8x8_t _w0 = vld1_s8(kptr);
-                int8x8_t _w1 = vld1_s8(kptr + 8);
-
-                int16x8_t _s0 = vmull_s8(_val0, _w0);
-                _s0 = vmlal_s8(_s0, _val1, _w1);
-
-                _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-
-                sptr += 2;
-                kptr += 16;
-            }
-            for (; i < num_input; i++)
-            {
-                int8x8_t _val = vdup_n_s8(sptr[0]);
-
-                int8x8_t _w = vld1_s8(kptr);
-
-                int16x8_t _s0 = vmull_s8(_val, _w);
-                _sum0 = vaddw_s16(_sum0, vget_low_s16(_s0));
-                _sum1 = vaddw_s16(_sum1, vget_high_s16(_s0));
-
-                sptr += 1;
-                kptr += 8;
-            }
-
-            int* outptr = (int*)top_blob_int32;
-            vst1q_s32(outptr + p * 8, _sum0);
-            vst1q_s32(outptr + p * 8 + 4, _sum1);
-        }
-    }
-#endif // __ARM_NEON
-
-    if (out_elempack == 1)
-    {
-        // num_output
-        #pragma omp parallel for num_threads(opt.num_threads)
-        for (int p = 0; p < num_output / out_elempack; p++)
-        {
-            const signed char* kptr = weight_data_int8.row<const signed char>(p);
-            const signed char* sptr = bottom_blob_int8_flattened;
-
-            int sum = 0;
-
-            int i = 0;
-            for (; i < num_input; i++)
-            {
-                signed char val = sptr[0];
-
-                signed char w = kptr[0];
-
-                sum += val * w;
-
-                sptr += 1;
-                kptr += 1;
-            }
-
-            int* outptr = (int*)top_blob_int32;
-            outptr[p] = sum;
-        }
-    }
-
-    Mat scale_data(num_output);
-    for (int p = 0; p < num_output; p++)
-    {
-        // dequantize
-        float scale_in;
-        if (weight_data_int8_scales[p] == 0)
-            scale_in = 0;
-        else
-            scale_in = 1.f / (bottom_blob_int8_scales[0] * weight_data_int8_scales[p]);
-
-        scale_data[p] = scale_in;
-    }
-
-    dequantize_from_int32(top_blob_int32, top_blob, scale_data, bias_data, opt);
-
-    if (activation)
-    {
-        activation->forward_inplace(top_blob, opt);
-    }
-
-    return 0;
-}
-#endif // NCNN_INT8
 
 } // namespace ncnn
