@@ -16,14 +16,14 @@
 
 #include "layer_type.h"
 
-#include "fused_activation.h"
-
 namespace ncnn {
 
 ConvolutionDepthWise::ConvolutionDepthWise()
 {
     one_blob_only = true;
     support_inplace = false;
+
+    use_int8_requantize = false;
 }
 
 int ConvolutionDepthWise::load_param(const ParamDict& pd)
@@ -55,12 +55,7 @@ int ConvolutionDepthWise::load_param(const ParamDict& pd)
 
     if (int8_scale_term)
     {
-#if NCNN_INT8
-        support_int8_storage = true;
-#else
-        NCNN_LOGE("please build ncnn with NCNN_INT8 enabled for int8 inference");
-        return -1;
-#endif
+        use_int8_inference = true;
     }
 
     return 0;
@@ -79,8 +74,7 @@ int ConvolutionDepthWise::load_model(const ModelBin& mb)
             return -100;
     }
 
-#if NCNN_INT8
-    if (int8_scale_term == 1 || int8_scale_term == 101)
+    if (int8_scale_term == 1)
     {
         weight_data_int8_scales = mb.load(group, 1);
         bottom_blob_int8_scales = mb.load(1, 1);
@@ -89,7 +83,7 @@ int ConvolutionDepthWise::load_model(const ModelBin& mb)
         bottom_blob_int8_scales = Mat(group);
         bottom_blob_int8_scales.fill(bottom_blob_int8_scale);
     }
-    else if (int8_scale_term == 2 || int8_scale_term == 102)
+    else if (int8_scale_term == 2)
     {
         weight_data_int8_scales = mb.load(1, 1);
         bottom_blob_int8_scales = mb.load(1, 1);
@@ -104,22 +98,11 @@ int ConvolutionDepthWise::load_model(const ModelBin& mb)
         bottom_blob_int8_scales.fill(bottom_blob_int8_scale);
     }
 
-    if (int8_scale_term > 100)
-    {
-        top_blob_int8_scales = mb.load(1, 1);
-
-        float top_blob_int8_scale = top_blob_int8_scales[0];
-        top_blob_int8_scales = Mat(group);
-        top_blob_int8_scales.fill(top_blob_int8_scale);
-    }
-#endif // NCNN_INT8
-
     return 0;
 }
 
 int ConvolutionDepthWise::create_pipeline(const Option& opt)
 {
-#if NCNN_INT8
     // runtime quantize the weight data
     if (opt.use_int8_inference && weight_data.elemsize == (size_t)4u && int8_scale_term)
     {
@@ -133,17 +116,14 @@ int ConvolutionDepthWise::create_pipeline(const Option& opt)
         {
             Option opt_q = opt;
             opt_q.blob_allocator = int8_weight_data.allocator;
-            opt_q.use_packing_layout = false;
 
             const Mat weight_data_g = weight_data.range(weight_data_size_g * g, weight_data_size_g);
             Mat int8_weight_data_g = int8_weight_data.range(weight_data_size_g * g, weight_data_size_g);
-            const Mat weight_data_int8_scales_g = weight_data_int8_scales.range(g, 1);
-            quantize_to_int8(weight_data_g, int8_weight_data_g, weight_data_int8_scales_g, opt_q);
+            quantize_float32_to_int8(weight_data_g, int8_weight_data_g, weight_data_int8_scales[g], opt_q);
         }
 
         weight_data = int8_weight_data;
     }
-#endif // NCNN_INT8
 
     return 0;
 }
@@ -153,12 +133,10 @@ int ConvolutionDepthWise::forward(const Mat& bottom_blob, Mat& top_blob, const O
     // convolv with NxN kernel
     // value = value + bias
 
-#if NCNN_INT8
     if (opt.use_int8_inference && weight_data.elemsize == (size_t)1u)
     {
         return forward_int8(bottom_blob, top_blob, opt);
     }
-#endif
 
     int w = bottom_blob.w;
     int h = bottom_blob.h;
@@ -241,7 +219,42 @@ int ConvolutionDepthWise::forward(const Mat& bottom_blob, Mat& top_blob, const O
                         sum += val * w;
                     }
 
-                    outptr[j] = activation_ss(sum, activation_type, activation_params);
+                    if (activation_type == 1)
+                    {
+                        sum = std::max(sum, 0.f);
+                    }
+                    else if (activation_type == 2)
+                    {
+                        float slope = activation_params[0];
+                        sum = sum > 0.f ? sum : sum * slope;
+                    }
+                    else if (activation_type == 3)
+                    {
+                        float min = activation_params[0];
+                        float max = activation_params[1];
+                        if (sum < min)
+                            sum = min;
+                        if (sum > max)
+                            sum = max;
+                    }
+                    else if (activation_type == 4)
+                    {
+                        sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                    }
+                    else if (activation_type == 5)
+                    {
+                        const float MISH_THRESHOLD = 20;
+                        float x = sum, y;
+                        if (x > MISH_THRESHOLD)
+                            y = x;
+                        else if (x < -MISH_THRESHOLD)
+                            y = expf(x);
+                        else
+                            y = logf(expf(x) + 1);
+                        sum = static_cast<float>(x * tanh(y));
+                    }
+
+                    outptr[j] = sum;
                 }
 
                 outptr += outw;
@@ -293,7 +306,42 @@ int ConvolutionDepthWise::forward(const Mat& bottom_blob, Mat& top_blob, const O
                             kptr += maxk;
                         }
 
-                        outptr[j] = activation_ss(sum, activation_type, activation_params);
+                        if (activation_type == 1)
+                        {
+                            sum = std::max(sum, 0.f);
+                        }
+                        else if (activation_type == 2)
+                        {
+                            float slope = activation_params[0];
+                            sum = sum > 0.f ? sum : sum * slope;
+                        }
+                        else if (activation_type == 3)
+                        {
+                            float min = activation_params[0];
+                            float max = activation_params[1];
+                            if (sum < min)
+                                sum = min;
+                            if (sum > max)
+                                sum = max;
+                        }
+                        else if (activation_type == 4)
+                        {
+                            sum = static_cast<float>(1.f / (1.f + exp(-sum)));
+                        }
+                        else if (activation_type == 5)
+                        {
+                            const float MISH_THRESHOLD = 20;
+                            float x = sum, y;
+                            if (x > MISH_THRESHOLD)
+                                y = x;
+                            else if (x < -MISH_THRESHOLD)
+                                y = expf(x);
+                            else
+                                y = logf(expf(x) + 1);
+                            sum = static_cast<float>(x * tanh(y));
+                        }
+
+                        outptr[j] = sum;
                     }
 
                     outptr += outw;
@@ -346,7 +394,6 @@ void ConvolutionDepthWise::make_padding(const Mat& bottom_blob, Mat& bottom_blob
     }
 }
 
-#if NCNN_INT8
 static inline signed char float2int8(float v)
 {
     int int32 = static_cast<int>(round(v));
@@ -376,31 +423,32 @@ int ConvolutionDepthWise::forward_int8(const Mat& bottom_blob, Mat& top_blob, co
     const int kernel_extent_w = dilation_w * (kernel_w - 1) + 1;
     const int kernel_extent_h = dilation_h * (kernel_h - 1) + 1;
 
-    Mat bottom_blob_int8 = bottom_blob;
+    Mat bottom_blob_unbordered = bottom_blob;
     if (elemsize != 1)
     {
+        bottom_blob_unbordered.create(w, h, channels, (size_t)1u, opt.workspace_allocator);
+        if (bottom_blob_unbordered.empty())
+            return -100;
+
         const int channels_g = channels / group;
 
-        Mat scales(channels);
+        // quantize, scale and round to nearest
+        #pragma omp parallel for num_threads(opt.num_threads)
+        for (int g = 0; g < group; g++)
         {
-            float* ps = scales;
-            for (int g = 0; g < group; g++)
-            {
-                float scale = bottom_blob_int8_scales[g];
-                for (int q = 0; q < channels_g; q++)
-                {
-                    *ps++ = scale;
-                }
-            }
-        }
+            Option opt_g = opt;
+            opt_g.num_threads = 1;
+            opt_g.blob_allocator = bottom_blob_unbordered.allocator;
 
-        Option opt_q = opt;
-        opt_q.blob_allocator = opt.workspace_allocator;
-        quantize_to_int8(bottom_blob, bottom_blob_int8, scales, opt_q);
+            const Mat bottom_blob_g = bottom_blob.channel_range(channels_g * g, channels_g);
+            Mat bottom_blob_int8_g = bottom_blob_unbordered.channel_range(channels_g * g, channels_g);
+
+            quantize_float32_to_int8(bottom_blob_g, bottom_blob_int8_g, bottom_blob_int8_scales[g], opt_g);
+        }
     }
 
     Mat bottom_blob_bordered;
-    make_padding(bottom_blob_int8, bottom_blob_bordered, opt);
+    make_padding(bottom_blob_unbordered, bottom_blob_bordered, opt);
     if (bottom_blob_bordered.empty())
         return -100;
 
@@ -432,7 +480,6 @@ int ConvolutionDepthWise::forward_int8(const Mat& bottom_blob, Mat& top_blob, co
     }
 
     // int8
-    bool use_int8_requantize = int8_scale_term > 100;
     size_t out_elemsize = use_int8_requantize ? 1u : 4u;
 
     top_blob.create(outw, outh, num_output, out_elemsize, opt.blob_allocator);
@@ -464,30 +511,51 @@ int ConvolutionDepthWise::forward_int8(const Mat& bottom_blob, Mat& top_blob, co
                         sum += val * w;
                     }
 
-                    float scale_in;
-                    if (weight_data_int8_scales[g] == 0)
-                        scale_in = 0;
-                    else
-                        scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
-
-                    float sumfp32 = sum * scale_in;
-
-                    if (bias_term)
-                        sumfp32 += bias_data[g];
-
-                    sumfp32 = activation_ss(sumfp32, activation_type, activation_params);
-
                     if (use_int8_requantize)
                     {
-                        // requantize
-                        float scale_out = top_blob_int8_scales[g];
+                        // requantize and relu
+                        float scale_in;
+                        if (weight_data_int8_scales[g] == 0)
+                            scale_in = 0;
+                        else
+                            scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
+
+                        float sumfp32 = sum * scale_in;
+
+                        if (bias_term)
+                            sumfp32 += bias_data[g];
+
+                        float scale_out = top_blob_int8_scale; //FIXME load param
+
                         signed char sums8 = float2int8(sumfp32 * scale_out);
+
+                        if (activation_type == 1)
+                        {
+                            sums8 = std::max(sums8, (signed char)0);
+                        }
+
                         outptr[0] = sums8;
                         outptr += 1;
                     }
                     else
                     {
-                        // dequantize
+                        // dequantize and relu
+                        float scale_in;
+                        if (weight_data_int8_scales[g] == 0)
+                            scale_in = 0;
+                        else
+                            scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
+
+                        float sumfp32 = sum * scale_in;
+
+                        if (bias_term)
+                            sumfp32 += bias_data[g];
+
+                        if (activation_type == 1)
+                        {
+                            sumfp32 = std::max(sumfp32, 0.f);
+                        }
+
                         ((float*)outptr)[0] = sumfp32;
                         outptr += 4;
                     }
@@ -537,30 +605,51 @@ int ConvolutionDepthWise::forward_int8(const Mat& bottom_blob, Mat& top_blob, co
                             kptr += maxk;
                         }
 
-                        float scale_in;
-                        if (weight_data_int8_scales[g] == 0)
-                            scale_in = 0;
-                        else
-                            scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
-
-                        float sumfp32 = sum * scale_in;
-
-                        if (bias_term)
-                            sumfp32 += bias_data[g * num_output_g + p];
-
-                        sumfp32 = activation_ss(sumfp32, activation_type, activation_params);
-
                         if (use_int8_requantize)
                         {
-                            // requantize
-                            float scale_out = top_blob_int8_scales[g];
+                            // requantize and relu
+                            float scale_in;
+                            if (weight_data_int8_scales[g] == 0)
+                                scale_in = 0;
+                            else
+                                scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
+
+                            float sumfp32 = sum * scale_in;
+
+                            if (bias_term)
+                                sumfp32 += bias_data[g * num_output_g + p];
+
+                            float scale_out = top_blob_int8_scale; //FIXME load param
+
                             signed char sums8 = float2int8(sumfp32 * scale_out);
+
+                            if (activation_type == 1)
+                            {
+                                sums8 = std::max(sums8, (signed char)0);
+                            }
+
                             outptr[0] = sums8;
                             outptr += 1;
                         }
                         else
                         {
-                            // dequantize
+                            // dequantize and relu
+                            float scale_in;
+                            if (weight_data_int8_scales[g] == 0)
+                                scale_in = 0;
+                            else
+                                scale_in = 1.f / (bottom_blob_int8_scales[g] * weight_data_int8_scales[g]);
+
+                            float sumfp32 = sum * scale_in;
+
+                            if (bias_term)
+                                sumfp32 += bias_data[g * num_output_g + p];
+
+                            if (activation_type == 1)
+                            {
+                                sumfp32 = std::max(sumfp32, 0.f);
+                            }
+
                             ((float*)outptr)[0] = sumfp32;
                             outptr += 4;
                         }
@@ -572,6 +661,5 @@ int ConvolutionDepthWise::forward_int8(const Mat& bottom_blob, Mat& top_blob, co
 
     return 0;
 }
-#endif // NCNN_INT8
 
 } // namespace ncnn
