@@ -25,9 +25,6 @@ public:
     Packing_x86();
 
     virtual int forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const;
-
-protected:
-    int forward_int8(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const;
 };
 
 } // namespace ncnn
