@@ -50,14 +50,8 @@ int waifu2x_getData(void*& out, unsigned long& outSize, double& tick, int& callB
     outSize = v.outSize;
 
     v.out = NULL;
-    double encodeTick = (v.encodeTick.time + v.encodeTick.millitm/1000.0) - (v.startTick.time + v.encodeTick.millitm / 1000.0);
-    double procTick = (v.procTick.time + v.procTick.millitm / 1000.0) - (v.encodeTick.time + v.encodeTick.millitm / 1000.0);
-    double decodeTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.procTick.time + v.procTick.millitm/1000.0);
-    double allTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.startTick.time + v.startTick.millitm / 10000.0);
 
-    waifu2x_printf(stdout, "[waifu2x] end encode imageId :%d, encode:%.2fs, proc:%.2fs, decode:%.2fs, \n",
-        v.callBack, encodeTick, procTick, decodeTick);
-    tick = allTick;
+    tick = v.allTick;
     return v.id;
 }
 
@@ -139,8 +133,8 @@ void* waifu2x_proc(void* args)
             int scale_run_count = 1;
             if (v.toH > 0 && v.toW > 0 && h > 0 && w > 0)
             {
-                scale_run_count = std::max(int(v.toW/ w), scale_run_count);
-                scale_run_count = std::max(int(v.toH/ h), scale_run_count);
+                scale_run_count = std::max(int(ceil(v.toW * 1.0/ w)), scale_run_count);
+                scale_run_count = std::max(int(ceil(v.toH * 1.0 / h)), scale_run_count);
                 scale_run_count = ceil(log(scale_run_count)/log(2));
                 scale_run_count = std::max(scale_run_count, 1);
             }
@@ -257,6 +251,13 @@ void* waifu2x_proc(void* args)
                 waifu2x_printf(stderr, "[waifu2x] encode image %d failed, %s\n", v.id, error);
             }
             ftime(&v.saveTick);
+            double encodeTick = (v.encodeTick.time + v.encodeTick.millitm / 1000.0) - (v.startTick.time + v.encodeTick.millitm / 1000.0);
+            double procTick = (v.procTick.time + v.procTick.millitm / 1000.0) - (v.encodeTick.time + v.encodeTick.millitm / 1000.0);
+            double decodeTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.procTick.time + v.procTick.millitm / 1000.0);
+            double allTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.startTick.time + v.startTick.millitm / 10000.0);
+            v.allTick = allTick;
+            waifu2x_printf(stdout, "[waifu2x] end encode imageId :%d, encode:%.2fs, proc:%.2fs, decode:%.2fs, \n",
+                v.callBack, encodeTick, procTick, decodeTick);
         }
         else
         {
